/// <reference types="node" />
import { Readable } from 'stream';
import { Logger } from './logger';
import { NexeOptions } from './options';
import { NexeTarget } from './target';
export interface NexeFile {
    filename: string;
    absPath: string;
    contents: string;
}
export declare class NexeCompiler {
    options: NexeOptions;
    private start;
    private env;
    private compileStep;
    log: Logger;
    src: string;
    files: NexeFile[];
    input: string;
    bundledInput?: string;
    output: string | null;
    resources: {
        bundle: string;
        index: {
            [key: string]: number[];
        };
    };
    readFileAsync: (file: string) => Promise<NexeFile>;
    writeFileAsync: (file: string, contents: Buffer | string) => Promise<void>;
    replaceInFileAsync: (file: string, replacer: any, replaceValue: string) => Promise<void>;
    setFileContentsAsync: (file: string, contents: string | Buffer) => Promise<void>;
    private nodeSrcBinPath;
    constructor(options: NexeOptions);
    quit(): Promise<{}>;
    getNodeExecutableLocation(target?: NexeTarget): string;
    private _runBuildCommandAsync(command, args);
    private _configureAsync();
    private _buildAsync();
    private _fetchPrebuiltBinaryAsync(target);
    private _generateHeader();
    private _serializeHeader(header);
    compileAsync(target: NexeTarget): Promise<NodeJS.ReadableStream | Readable>;
    private _assembleDeliverable(header, binary);
}
